#if !defined  HAVE_DEBRUIJN_H__
#define       HAVE_DEBRUIJN_H__


#include "fxttypes.h"
#include "comb/necklace.h"
#include "bits/tinyfactors.h"


class debruijn : public necklace
// (lexicographic minimal) De Bruijn sequence
// Implementation following Knuth
{
public:
    ulong nt_, k_;
    ulong dq_;
    ulong tfb_; // tiny factor bits

public:
    debruijn(ulong m, ulong n)
        : necklace(m, n)
    {
        first();
    }

    ~debruijn()  { ; }

    void first()
    {
        necklace::first();
        nt_ = 1;
        k_ = 0;
        dq_ = 0;
        tfb_ = tiny_factors_tab[n_];
    }

    int done()  const  { return  dq_; }

    ulong next()  // return new element (\in 0..m-1)
    {
        if ( 0==nt_ )  next_string();

        --nt_;
        ++k_;
        return  a_[ k_ ];
    }

    void next_string()  // make new string
    {
        do
        {
            necklace::next();
            if ( 0==j_ )  { dq_ = 1;  return; }
        }
//        while ( (n_/j_)*j_ != n_ );  // slow
//        while ( ! is_tiny_factor(n_, j_) );  // faster
        while ( 0 == ( (tfb_>>j_) & 1 ) );  // fastest

        nt_ = j_;
        k_ = 0;
    }

    void make_seq(ulong *f, ulong n)
    {
        first();
        for (ulong k=0; k<n; ++k)  f[k] = next();
    }
};
// -------------------------


#endif  // !defined HAVE_DEBRUIJN_H__
